#!/bin/bash -e

#    Copyright (C) 2013 Andrey Uzunov
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.

if [ "1" -ne "$#" ]
then
        echo "usage: pushproxy.sh raw-access-log"
        exit

fi

source $(dirname $0)/include/common.sh

FROM=`date +"%Y%m%d" -d "$FIRSTLINE_DATE"`
TO=`date +"%Y%m%d" -d "$LASTLINE_DATE"`

if [ "$FROM" -gt "$TO" ]
then
  writelog "First date is greater than the last one. EXIT"
  exit
fi

while [ "$FROM" -le "$TO" ]
do
  LOG_DATE=`date +"%d/%b/%Y" -d "$FROM"`
  
  writelog "Trying $LOG_DATE"
  
  LOG_DATE_FOR_NAME=`date +"%Y-%m-%d" -d "$FROM"`
  
  FILTERED_AND_SORTED="temp-${BASE_INPUT}-${LOG_DATE_FOR_NAME}-sorted"
  
  FROM=`date +"%Y%m%d" -d "$FROM + 1 day"`
  
  LOG_DATE=" \[$LOG_DATE:"
  
  #separate by date
  #filter everything but GET and POST
  #filter buggy requests and error responses

  writelog "Fast filter and sort..."
  
  set +e
  grep "$LOG_DATE" "$1" |\
  egrep "\"((GET)|(POST)) " |\
  grep -v '" 400 ' |\
  egrep -v '] "([^\ ]+) ([^\ ]+)" ' > "$FILTERED_AND_SORTED"
  set -e

  loglines "$FILTERED_AND_SORTED"
  
  if [ "$LINES" -ge "1" ]
  then
  
    REORDERED="$FILTERED_AND_SORTED"
    
    #browsers and is_mobile option
    for BROWSERS in 0
    do
      #pageload timeout option
      for PL_TIMEOUT in 1000
      do
        #session timeout option
        for SESS_TIMEOUT in 180000 3600000
        do
          #size limit
          for SIZE in -1 1000 10000
          do
            #using history per user
            for HISTORY in 0 1
            do
              #different thresholds
              for THRESHOLD in 0 0.1 0.2 0.3 0.4 0.5 0.6 0.7 0.8 0.9 1
              do
                #different algos
                for ALGO in basic basic-less basic-more
                do
                  OUTPUT_OPTION="1"
                  
                  if [ ! -f "$ALGOS_LOG" ]
                  then
                    #write CSV header
                    writelog "CSV header will be written"
                    OUTPUT_OPTION="2"
                  fi
                  
                  #echo "${ALGO}.php ${SESS_TIMEOUT} ${PL_TIMEOUT} 20000 ${REORDERED} 1 $THRESHOLD -1 ${BROWSERS} ${OUTPUT_OPTION} ${SIZE} ${HISTORY}"
                  run_test "${ALGO}.php ${SESS_TIMEOUT} ${PL_TIMEOUT} 20000 ${REORDERED} 1 $THRESHOLD -1 ${BROWSERS} ${OUTPUT_OPTION} ${SIZE} ${HISTORY} 1"
                done
              done
            done
          done
        done
      done
    done
    rm "$FILTERED_AND_SORTED"
  else
    rm "$FILTERED_AND_SORTED"
  fi
done

#if ls plot-*.png &> /dev/null;
#then
#  TARFILE="plots-${COMMON_FILENAME}.tar.gz"
#
#  writelog "Writing plots to ${TARFILE}..."
#
#  tar -zcvf "$TARFILE" plot-*.png > /dev/null
#
#  rm plot-*.png
#fi

writelog "SUCCESS. EXIT"

exit 
